from binance.spot import Spot
from Utils import DataHelpers
import logging
from binance.error import ClientError
from binance_f import RequestClient
from binance_f.model.constant import *
import time

class BinanceExchange():
    def __init__(self, credentials, type='SPOT', sandbox=False):
        self.credentials = credentials
        self.sandbox = sandbox

        if type == 'SPOT':
            if sandbox:
                self.client = Spot(key=credentials['spot']['key'], secret=credentials['spot']['secret'], base_url='https://testnet.binance.vision')
                self.futuresClient = RequestClient(api_key=credentials['futures']['key'], secret_key=credentials['futures']['secret'], url='https://testnet.binancefuture.com')
            else:
                self.client = Spot(key=credentials['spot']['key'], secret=credentials['spot']['secret'])

        self.timeIntervlas = ['1m', '3m', '5m', '15m', '30m', '1h', '2h', '4h', '6h', '8h', '12h', '1d', '3d', '1w', '1M']

    def fetchBalance(self, asset=''):
        try:
            balances = self.client.account()['balances']
        except Exception:
            return None

        if asset == '':
            return balances
        else:
            for balance in balances:
                if balance['asset'] == asset:
                    return balance
        return None

    def SymbolTradeHistory(self, symbol):
        try:
            return self.client.my_trades(symbol)
        except Exception:
            return None

    @staticmethod
    def isOrderDataValid(order : DataHelpers.OrderData):
        if order.orderType not in ['LIMIT', 'MARKET', 'STOP_LOSS', 'STOP_LOSS_LIMIT', 'TAKE_PROFIT', 'TAKE_PROFIT_LIMIT', 'LIMIT_MAKER']:
            return False

        if order.side not in ['BUY', 'SELL']:
            return False
        
        if order.newOrderRespType not in [None, 'ACK', 'RESULT', 'FULL']:
            return False
        
        if order.timeInForce not in [None, 'GTC', 'IOC', 'FOK']:
            return False
            
        if order.orderType == 'LIMIT':
            if not (order.timeInForce is None or order.quantity is None or order.price is None):
                return True

        elif order.orderType == 'MARKET':
            if not (order.quantity is None and order.quoteOrderQty is None):
                return True

        elif order.orderType == 'STOP_LOSS':
            if not (order.quantity is None or order.stopPrice is None):
                return True

        elif order.orderType == 'STOP_LOSS_LIMIT':
            if not (order.timeInForce is None or order.quantity is None or order.price is None or order.stopPrice is None):
                return True

        elif order.orderType == 'TAKE_PROFIT':
            if not (order.quantity is None or order.stopPrice is None):
                return True

        elif order.orderType == 'TAKE_PROFIT_LIMIT':
            if not (order.timeInForce is None or order.quantity is None or order.price is None or order.stopPrice is None):
                return True

        elif order.orderType == 'LIMIT_MAKER':
            if not (order.quantity is None or order.price is None):
                return True
        
        return False

    @staticmethod
    def isFuturesOrderDataValid(order : DataHelpers.futuresOrderData):
        if order.side not in ['BUY', 'SELL']:
            return False

        if order.orderType not in ['LIMIT', 'MARKET', 'STOP', 'STOP_MARKET', 'TAKE_PROFIT', 'TAKE_PROFIT_MARKET', 'TRAILING_STOP_MARKET']:
            return False

        if order.positionSide not in [None, 'BOTH', 'LONG', 'SHORT']:
            return False

        if order.timeInForce not in [None, 'GTC', 'IOC', 'FOK', 'GTX']:
            return False

        if order.workingType not in [None, 'MARK_PRICE', 'CONTRACT_PRICE']:
            return False
        
        if order.newOrderRespType not in [None, 'ACK', 'RESULT']:
            return False

        if order.closePosition not in [True, False]:
            return False

        if not order.callbackRate is None:
            if not (0.1 <= order.callbackRate <= 5):
                return False

        if order.priceProtect not in [True, False]:
            return False

        if order.closePosition == True and order.quantity is not None:
            return False

        if order.reduceOnly not in [True, False]:
            return False

        if order.closePosition == True and order.reduceOnly is True:
            return False


        if order.orderType == 'LIMIT':
            if not (order.timeInForce is None or order.quantity is None or order.price is None):
                return True

        elif order.orderType == 'MARKET':
            if order.quantity is not None:
                return True

        elif order.orderType in ['STOP', 'TAKE_PROFIT']:
            if not (order.quantity is None or order.price is None or order.stopPrice is None):
                return True

        elif order.orderType in ['STOP_MARKET', 'TAKE_PROFIT_MARKET']:
            if order.stopPrice is not None:
                return True

        elif order.orderType == 'TRAILING_STOP_MARKET':
            if order.callbackRate is not None:
                return True

    @staticmethod
    def getOrderAsDict(order : DataHelpers.OrderData):
        if order.timestamp is None:
            raise Exception('Timestamp must be set')

        params = {}
        params['symbol'] = order.symbol
        params['side'] = order.side
        params['type'] = order.orderType
        params['timestamp'] = order.timestamp

        if not order.timeInForce is None:
            params['timeInForce'] = order.timeInForce

        if not order.quantity is None:
            params['quantity'] = order.quantity
        
        if not order.quoteOrderQty is None:
            params['quoteOrderQty'] = order.quoteOrderQty

        if not order.price is None:
            params['price'] = order.price
        
        if not order.newOrderRespType is None:
            params['newOrderRespType'] = order.newOrderRespType
        
        if not order.stopPrice is None:
            params['stopPrice'] = order.stopPrice
        
        if not order.icebergQty is None:
            params['icebergQty'] = order.icebergQty
        
        if not order.newClientOrderId is None:
            params['newOrderRespType'] = order.newOrderRespType
        
        if not order.recvWindow is None:
            params['recvWindow'] = order.recvWindow

        return params

    @staticmethod
    def getFuturesOrderAsDict(order : DataHelpers.futuresOrderData):
        params = {}
        params['symbol'] = order.symbol
        params['side'] = order.side
        params['ordertype'] = order.orderType

        if not order.positionSide is None:
            params['positionSide'] = order.positionSide

        if not order.timeInForce is None:
            params['timeInForce'] = order.timeInForce

        if not order.quantity is None:
            params['quantity'] = order.quantity
        
        if not order.reduceOnly is None:
            params['reduceOnly'] = order.reduceOnly

        if not order.price is None:
            params['price'] = order.price
        
        if not order.newClientOrderId is None:
            params['newClientOrderId'] = order.newClientOrderId
        
        if not order.stopPrice is None:
            params['stopPrice'] = order.stopPrice
        
        if not order.closePosition is None:
            params['closePosition'] = order.closePosition
        
        if not order.activationPrice is None:
            params['activationPrice'] = order.activationPrice
        
        if not order.callbackRate is None:
            params['callbackRate'] = order.callbackRate

        if not order.workingType is None:
            params['workingType'] = order.workingType

        if not order.priceProtect is None:
            params['priceProtect'] = order.priceProtect

        if not order.newOrderRespType is None:
            params['newOrderRespType'] = order.newOrderRespType

        if not order.recvWindow is None:
            params['recvWindow'] = order.recvWindow

        return params

    def testOrder(self, orderData):
        orderData.setTimestamp()
        params = self.getOrderAsDict(orderData)

        try:
            response = self.client.new_order_test(**params)
            logging.info(response)
            return response
        except ClientError as error:
            logging.error(
                "Found error. status: {}, error code: {}, error message: {}".format(
                    error.status_code, error.error_code, error.error_message
                )
            )

    def makeOrder(self, orderData):
        params = self.getOrderAsDict(orderData)

        try:
            response = self.client.new_order(**params)
            logging.info(response)
            return response
        except ClientError as error:
            logging.error(
                "Found error. status: {}, error code: {}, error message: {}".format(
                    error.status_code, error.error_code, error.error_message
                )
            )

    def getSymbolOrders(self, symbol):
        try:
            return self.client.get_orders(symbol, timestamp=time.time())
        except Exception:
            return None

    def getOpenOrders(self, symbol=None):
        try:
            return self.client.get_open_orders(symbol, timestamp=time.time())
        except Exception:
            return None

    def cancelAllSymbolOpenOrders(self, symbol):
        return self.client.cancel_open_orders(symbol, timestamp=time.time())

    def cancelSymbolOpenOrder(self, symbol, orderId=None, localOrderId=None):
        if not orderId is None:
            return self.client.cancel_order(symbol, orderId=orderId, timestamp=time.time())
        elif not localOrderId is None:
            return self.client.cancel_order(symbol, origClientOrderId=localOrderId, timestamp=time.time())
        else:
            raise Exception('Specify either order Id in the exchange or local Id sent with the order')
    
    def getOrder(self, symbol, orderId=None, localOrderId=None):
        if not orderId is None:
            return self.client.get_order(symbol, orderId=orderId, timestamp=time.time())
        elif not localOrderId is None:
            return self.client.get_order(symbol, origClientOrderId=localOrderId, timestamp=time.time())
        else:
            raise Exception('Specify either order Id in the exchange or local Id sent with the order')
        
    def getTradingFees(self):
        try:
            return self.client.trade_fee()
        except Exception:
            return None

    def getSymbolAveragePrice(self, symbol):
        try:
            return self.client.avg_price(symbol)
        except Exception:
            return None

    def getSymbolLatestTrades(self, symbol, limit=None):
        try:
            if not limit is None:
                if limit > 1000: limit = 1000
                elif limit < 1: limit = 1

                return self.client.trades(symbol, limit)
            else:
                return self.client.trades(symbol)
        except Exception:
            return None

    def getSymbolTickerPrice(self, symbol):
        try:
            return self.client.ticker_price(symbol)['price']
        except Exception:
            return None

    def getSymbolKlines(self, symbol, interval, startTime=None, endTime=None, limit=None):
        if not interval in self.timeIntervlas:
            raise Exception('Time interval is not valid.')
        try:
            return self.client.klines(symbol, interval, startTime=startTime, endTime=endTime, limit=limit)
        except Exception:
            return None

    def getExchangeTime(self):
        try:
            return self.client.time()
        except Exception:
            return None

    def getSymbol24hTicker(self, symbol):
        try:
            return self.client.ticker_24hr(symbol)
        except Exception:
            return None

    def getSymbolFuturesOrders(self, symbol):
        return self.futuresClient.get_all_orders(symbol=symbol)

    def getFuturesBalance(self):
        return self.futuresClient.get_balance()

    def makeFuturesOrder(self, futuresOrderData):
        params = self.getFuturesOrderAsDict(futuresOrderData)

        try:
            response = self.futuresClient.post_order(**params)
            logging.info(response)
            return response
        except ClientError as error:
            logging.error(
                "Found error. status: {}, error code: {}, error message: {}".format(
                    error.status_code, error.error_code, error.error_message
                )
            )

    def cancelAllSymbolFuturesOpenOrders(self, symbol):
        return self.futuresClient.cancel_all_orders(symbol=symbol)

    def cancelFuturesOrder(self, symbol, orderId=None, localOrderId=None):
        if not orderId is None:
            return self.futuresClient.cancel_order(symbol, orderId=orderId)
        elif not localOrderId is None:
            return self.futuresClient.cancel_order(symbol, origClientOrderId=localOrderId)
        else:
            raise Exception('Specify either order Id in the exchange or local Id sent with the order')

    def getAllFuturesOpenOrders(self, symbol=None):
        return self.futuresClient.get_open_orders(symbol=symbol)
        
    def getFuturesOrder(self, symbol, orderId=None, localOrderId=None):
        if not orderId is None:
            return self.futuresClient.get_order(symbol, orderId=orderId)
        elif not localOrderId is None:
            return self.futuresClient.get_order(symbol, origClientOrderId=localOrderId)
        else:
            raise Exception('Specify either order Id in the exchange or local Id sent with the order')

    def cancellAllSymbolFuturesOrders(self, symbol, countdownTime):
        return self.futuresClient.auto_cancel_all_orders(symbol, countdownTime)

    def changeInitialLeverage(self, symbol, leverage):
        return self.futuresClient.change_initial_leverage(symbol=symbol, leverage=leverage)

    def changeMarginType(self, symbol, marginType):
        if marginType not in ['ISOLATED', 'CROSSED']:
            raise Exception('Margin type specified is not acceptable')
        
        return self.futuresClient.change_margin_type(symbol=symbol, marginType=marginType)

    def changePositionMargin(self, symbol, amount, marginType):
        if marginType not in [1, 2]:
            raise Exception('Bad type specified.')
        self.futuresClient.change_position_margin(symbol=symbol, amount=amount, type=marginType)

    def getPosition(self):
        return self.futuresClient.get_position()

    def spotBestBidAsks(self, symbol=None):
        return self.client.book_ticker(symbol=symbol)

    def getSpotSymbolOrderBook(self, symbol, limit=None):
        if limit is None:
            return self.client.depth(symbol)
        else:
            return self.clinet.depth(symbol, limit=limit)

    def getSpotRecentSymbolTrades(self, symbol, limit=None):
        if limit is None:
            return self.client.trades(symbol)
        else:
            return self.clinet.trades(symbol, limit=limit)

    def getFutureSymbolOrderBook(self, symbol, limit=None):
        if limit is None:
            return self.futuresClient.get_order_book(symbol=symbol)
        else:
            return self.futuresClient.get_order_book(symbol=symbol, limit=limit)

    def getFutureSymbolRecentOrders(self, symbol, limit=None):
        if limit is None:
            return self.futuresClient.get_recent_trades_list(symbol=symbol)
        else:
            return self.futuresClient.get_recent_trades_list(symbol=symbol, limit=limit)


import json
from Exchanges import BinanceExchange
from Utils import DataHelpers


class TradeGate():
    def __init__(self, configDict, exchangeName, spot=False, sandbox=False):
        exchangeClass = self.getCorrectExchange(exchangeName)
        if sandbox:
            self.apiKey = configDict['credentials']['test']['spot']['key']
            self.apiSecret = configDict['credentials']['test']['spot']['secret']

            self.exchange = exchangeClass(configDict['credentials']['test'], type='SPOT', sandbox=True)
        else:
            self.apiKey = configDict['credentials']['main']['spot']['key']
            self.apiSecret = configDict['credentials']['main']['spot']['secret']

            self.exchange = exchangeClass(configDict['credentials']['test'], type='SPOT', sandbox=False)

    def getBalance(self, asset=''):
        return self.exchange.fetchBalance(asset)

    def getSymbolTradeHistory(self, symbol):
        return self.exchange.SymbolTradeHistory(symbol)

    @staticmethod
    def getCorrectExchange(exchangeName):
        if exchangeName == 'Binance':
            return BinanceExchange.BinanceExchange

    def createAndTestOrder(self, symbol, side, orderType, quantity=None, price=None, timeInForce=None, stopPrice=None, icebergQty=None, newOrderRespType=None, recvWindow=None,
                            newClientOrderId=None):
        currOrder = DataHelpers.OrderData(symbol.upper(), side.upper(), orderType.upper())

        if not quantity is None:
            currOrder.setQuantity(quantity)

        if not price is None:
            currOrder.setPrice(price)

        if not timeInForce is None:
            currOrder.setTimeInForce(timeInForce)

        if not stopPrice is None:
            currOrder.setStopPrice(stopPrice)

        if not icebergQty is None:
            currOrder.setIcebergQty(icebergQty)

        if not newOrderRespType is None:
            currOrder.setNewOrderRespType(newOrderRespType)
        
        if not recvWindow is None:
            currOrder.setRecvWindow(recvWindow)

        if not newClientOrderId is None:
            currOrder.setNewClientOrderId(newClientOrderId)

        if not self.exchange.isOrderDataValid(currOrder):
            raise Exception('Incomplete data provided.')

        self.exchange.testOrder(currOrder)

        return currOrder

    def makeOrder(self, orderData):
        return self.exchange.makeOrder(orderData)

    def getSymbolOrders(self, symbol):
        return self.exchange.getSymbolOrders(symbol)

    def getOpenOrders(self, symbol=None):
        return self.exchange.getOpenOrders(symbol)

    def getOrder(self, symbol, orderId=None, localOrderId=None):
        return self.exchange.getOrder(symbol, orderId, localOrderId)

    def cancelAllSymbolOpenOrders(self, symbol):
        return self.exchange.cancelAllSymbolOpenOrders(symbol)

    def cancelSymbolOpenOrder(self, symbol, orderId=None, localOrderId=None):
        return self.exchange.cancelSymbolOpenOrder(symbol, orderId, localOrderId)

    def getTradingFees(self):
        return self.exchange.getTradingFees()

    def getSymbolAveragePrice(self, symbol):
        return self.exchange.getSymbolAveragePrice(symbol)
        
    def getSymbolLatestTrades(self, symbol, limit=None):
        return self.exchange.getSymbolLatestTrades(symbol, limit)

    def getSymbolTickerPrice(self, symbol):
        return self.exchange.getSymbolTickerPrice(symbol)

    def getSymbolKlines(self, symbol, interval, startTime=None, endTime=None, limit=None):
        return self.exchange.getSymbolKlines(symbol, interval, startTime, endTime, limit)

    def getExchangeTime(self):
        return self.exchange.getExchangeTime()

    def getSymbolFuturesOrders(self, symbol):
        return self.exchange.getSymbolFuturesOrders(symbol)

    def getFuturesBalance(self):
        return self.exchange.getFuturesBalance()

    def createAndTestFuturesOrder(self, symbol, side, orderType, positionSide=None, timeInForce=None, quantity=None, reduceOnly=False, price=None, newClientOrderId=None,
                                    stopPrice=None, closePosition=False, activationPrice=None, callbackRate=None, workingType=None, priceProtect=False, newOrderRespType=None,
                                    recvWindow=None):
        currOrder = DataHelpers.futuresOrderData(symbol.upper(), side.upper(), orderType.upper())

        if not positionSide is None:
            currOrder.setPositionSide(positionSide)
        
        if not timeInForce is None:
            currOrder.setTimeInForce(timeInForce)

        if not quantity is None:
            currOrder.setQuantity(quantity)

        if not reduceOnly is None:
            currOrder.setReduceOnly(reduceOnly)

        if not price is None:
            currOrder.setPrice(price)

        if not newClientOrderId is None:
            currOrder.setNewClientOrderId(newClientOrderId)

        if not stopPrice is None:
            currOrder.setStopPrice(stopPrice)

        if not closePosition is None:
            currOrder.setClosePosition(closePosition)

        if not activationPrice is None:
            currOrder.setActivationPrice(activationPrice)

        if not callbackRate is None:
            currOrder.setCallbackRate(callbackRate)

        if not workingType is None:
            currOrder.setWorkingType(workingType)

        if not priceProtect is None:
            currOrder.setPriceProtect(priceProtect)

        if not newOrderRespType is None:
            currOrder.setNewOrderRespType(newOrderRespType)
        
        if not recvWindow is None:
            currOrder.setRecvWindow(recvWindow)

        if not self.exchange.isFuturesOrderDataValid(currOrder):
            raise Exception('Incomplete data provided.')

        return currOrder

    def makeFuturesOrder(self, futuresOrderData):
        return self.exchange.makeFuturesOrder(futuresOrderData)

    def cancelAllSymbolFuturesOpenOrders(self, symbol):
        return self.exchange.cancelAllSymbolFuturesOpenOrders(symbol)

    def cancelFuturesOrder(self, symbol, orderId=None, localOrderId=None):
        return self.exchange.cancelFuturesOrder(symbol, orderId, localOrderId)

    def getAllFuturesOpenOrders(self, symbol=None):
        return self.exchange.getAllFuturesOpenOrders(symbol)

    def getFuturesOrder(self, symbol, orderId=None, localOrderId=None):
        return self.exchange.getFuturesOrder(symbol, orderId, localOrderId)

    def cancellAllSymbolFuturesOrders(self, symbol, countdownTime):
        return self.exchange.cancellAllSymbolFuturesOrders(symbol, countdownTime)

    def getAllOpenFuturesOrders(self, symbol=None):
        return self.exchange.getAllOpenFuturesOrders(symbol)

    def changeInitialLeverage(self, symbol, leverage):
        return self.exchange.changeInitialLeverage(symbol, leverage)

    def changeMarginType(self, symbol, marginType):
        return self.exchange.changeMarginType(symbol, marginType)

    def changePositionMargin(self, symbol, amount, marginType):
        return self.exchange.changePositionMargin(symbol, amount, marginType)

    def getPosition(self):
        return self.exchange.getPosition()

    def spotBestBidAsks(self, symbol=None):
        return self.exchange.spotBestBidAsks(symbol)

    def getSpotSymbolOrderBook(self, symbol, limit=None):
        return self.exchange.getSpotOrderBook(symbol, limit)

    def getSpotRecentSymbolTrades(self, symbol, limit=None):
        return self.exchange.getSpotRecentSymbolTrades(symbol, limit)

    def getFutureSymbolOrderBook(self, symbol, limit=None):
        return self.exchange.getFutureSymbolOrderBook(symbol, limit)

    def getFutureSymbolRecentOrders(self, symbol, limit=None):
        return self.exchange.getFutureSymbolRecentOrders(symbol, limit)



import unittest
import json
from TradeGates.TradeGate import TradeGate
import logging


class BinanceAccountInfoTest(unittest.TestCase):
    def setUp(self):
        with open('./config.json') as f:
            config = json.load(f)

        self.tradeGate = TradeGate(config['Binance'], 'Binance', sandbox=True)
        loglevel = logging.INFO
        logging.basicConfig(level=loglevel)
        self.log = logging.getLogger(__name__)

    @unittest.skip
    def testTradingFees(self):
        # self.log.info('\nTrading Fees: {}'.format(self.tradeGate.getTradingFees()))
        self.assertIsNotNone(self.tradeGate.getTradingFees())

    def testAveragePrice(self):
        # self.log.info('\BTCUSDT Average Price: {}'.format(self.tradeGate.getSymbolAveragePrice('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolAveragePrice('BTCUSDT'))

    def testLatestTrades(self):
        # self.log.info('\nLatest Trades For BTCUSDT: {}'.format(self.tradeGate.getSymbolLatestTrades('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolLatestTrades('BTCUSDT'))

    def testTickerPrice(self):
        # self.log.info('\n"BTCUSDT" Ticker Price: {}'.format(self.tradeGate.getSymbolTickerPrice('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolTickerPrice('BTCUSDT'))
    
    def testKlines(self):
        # self.log.info('\n"BTCUSDT" Ticker Price: {}'.format(self.tradeGate.getSymbolKlines('BTCUSDT', '1m', limit=10)))
        self.assertIsNotNone(self.tradeGate.getSymbolKlines('BTCUSDT', '1m', limit=10))

    def testExchangeTime(self):
        self.assertIsNotNone(self.tradeGate.getExchangeTime())


if __name__ == '__main__':
    unittest.main()


import unittest
import json
from TradeGates.TradeGate import TradeGate
import logging


class BinanceAccountInfoTest(unittest.TestCase):
    def setUp(self):
        with open('./config.json') as f:
            config = json.load(f)

        self.tradeGate = TradeGate(config['Binance'], 'Binance', sandbox=True)
        loglevel = logging.INFO
        logging.basicConfig(level=loglevel)
        self.log = logging.getLogger(__name__)

    def testFullBalance(self):
        # self.log.info('\nFull Balance: {}'.format(self.tradeGate.getBalance()))
        self.assertIsNotNone(self.tradeGate.getBalance(), 'Assert fetching balance is not none')

    def testSingleCoinBalance(self):
        # self.log.info('\nCoin Balance: {}'.format(self.tradeGate.getBalance('BTC')))
        self.assertIsNotNone(self.tradeGate.getBalance('BTC'), 'Assert fetching single coin balance is not none')

    def testTradeHistory(self):
        # self.log.info('\nHistory: {}'.format(self.tradeGate.getSymbolTradeHistory('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolTradeHistory('BTCUSDT'), 'Assert trade history not none')


if __name__ == '__main__':
    unittest.main()


import unittest
import json
from TradeGates.TradeGate import TradeGate
from Utils.DataHelpers import OrderData
import logging


class BinanceOrderingTest(unittest.TestCase):
    def setUp(self):
        with open('./config.json') as f:
            config = json.load(f)

        self.tradeGate = TradeGate(config['Binance'], 'Binance', sandbox=True)
        loglevel = logging.INFO
        logging.basicConfig(level=loglevel)
        self.log = logging.getLogger(__name__)

    def testNewTestOrder(self):
        try:
            res = self.tradeGate.createAndTestOrder('BTCUSDT', 'SELL', 'LIMIT', timeInForce='GTC', quantity=0.002, price=49500)
        except Exception as e:
            self.fail('Problem in order data')

    def testNewTestOrderBadOrderType(self):
        try:
            res = self.tradeGate.createAndTestOrder('BTCUSDT', 'SELL', 'LINIT', timeInForce='GTC', quantity=0.002, price=49500)
        except Exception as e:
            return
        
        self.fail('Problem in validating order data')

    def testNewOrder(self):
        try:
            verifiedOrder = self.tradeGate.createAndTestOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000, timeInForce='GTC')
        except Exception as e:
            self.fail('Problem in order data: {}'.format(str(e)))

        try:
            result = self.tradeGate.makeOrder(verifiedOrder)
            # self.log.info('\nOrder status: {}'.format(result))
        except Exception as e:
            self.fail('Problem in making order: {}'.format(str(e)))

    def testGetOrders(self):
        # self.log.info('\nOrders: {}'.format(self.tradeGate.getSymbolOrders('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolOrders('BTCUSDT'), 'Problem in getting list of all orders')

    def testGetOpenOrders(self):
        # self.log.info('\nOrders: {}'.format(self.tradeGate.getOpenOrders('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getOpenOrders(), 'Problem in getting list of open orders without symbol.')
        self.assertIsNotNone(self.tradeGate.getOpenOrders('BTCUSDT'), 'Problem in getting list of open orders with symbol.')

    def testGetOrder(self):
        try:
            verifiedOrder = self.tradeGate.createAndTestOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000, timeInForce='GTC')
            result = self.tradeGate.makeOrder(verifiedOrder)
        except Exception as e:
            self.fail('Problem in making order: {}'.format(str(e)))
        
        order = self.tradeGate.getOrder('BTCUSDT', orderId=result['orderId'])
        self.assertEqual(order['clientOrderId'], result['clientOrderId'])

        order = self.tradeGate.getOrder('BTCUSDT', localOrderId=result['clientOrderId'])
        self.assertEqual(order['orderId'], result['orderId'])

        self.tradeGate.cancelSymbolOpenOrder('BTCUSDT', orderId=result['orderId'])

    def testCancelingAllOpenOrders(self):
        self.testNewOrder()
        result = self.tradeGate.cancelAllSymbolOpenOrders('BTCUSDT')
        # self.log.info('\n{}'.format(result))

        openOrders = self.tradeGate.getOpenOrders('BTCUSDT')
        self.assertEqual(len(openOrders), 0, 'Problem in canceling all Open Orders')

    def testCancelingOrder(self):
        try:
            verifiedOrder = self.tradeGate.createAndTestOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000, timeInForce='GTC')
            result = self.tradeGate.makeOrder(verifiedOrder)
        except Exception as e:
            self.fail('Problem in making order: {}'.format(str(e)))
        result = self.tradeGate.cancelSymbolOpenOrder(symbol='BTCUSDT', orderId=result['orderId'])
        self.assertEqual(result['status'], 'CANCELED', 'Problem in canceling specified Open Orders')

        try:
            verifiedOrder = self.tradeGate.createAndTestOrder('BTCUSDT', 'BUY', 'LIMIT', quantity=0.002, price=35000, timeInForce='GTC')
            result = self.tradeGate.makeOrder(verifiedOrder)
        except Exception as e:
            self.fail('Problem in making order: {}'.format(str(e)))
        result = self.tradeGate.cancelSymbolOpenOrder(symbol='BTCUSDT', localOrderId=result['clientOrderId'])
        self.assertEqual(result['status'], 'CANCELED', 'Problem in canceling specified Open Orders')


if __name__ == '__main__':
    unittest.main()


import unittest
import json
from TradeGates.TradeGate import TradeGate
import logging


class BinanceFuturesTest(unittest.TestCase):
    def setUp(self):
        with open('./config.json') as f:
            config = json.load(f)

        self.tradeGate = TradeGate(config['Binance'], 'Binance', sandbox=True)
        loglevel = logging.INFO
        logging.basicConfig(level=loglevel)
        self.log = logging.getLogger(__name__)

    def testSymbolFuturesOrders(self):
        # self.log.info('\BTCUSDT Futures Orders: {}'.format(self.tradeGate.getAllFuturesOrders('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getSymbolFuturesOrders('BTCUSDT'))

    def testFuturesBalance(self):
        # self.log.info('\BTCUSDT Futures Balance: {}'.format(self.tradeGate.getFuturesBalance()))
        self.assertIsNotNone(self.tradeGate.getFuturesBalance())

    def testFuturesOrder(self):
        futuresOrderData = self.tradeGate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.002)

        result = self.tradeGate.makeFuturesOrder(futuresOrderData)
        self.log.info('\nFutures Order Result: {}'.format(result))

        self.assertIsNotNone(result, 'Problem in submiting futures order.')

    def testCancelingAllFututresOpenOrders(self):
        result = self.tradeGate.cancelAllSymbolFuturesOpenOrders('BTCUSDT')
        self.assertIsNotNone(result, 'Problem in canceling all futures orders')

    def testGetFuturesOpenOrders(self):
        # self.log.info('\nOrders: {}'.format(self.tradeGate.getOpenOrders('BTCUSDT')))
        self.assertIsNotNone(self.tradeGate.getAllFuturesOpenOrders(), 'Problem in getting list of open orders without symbol.')
        self.assertIsNotNone(self.tradeGate.getAllFuturesOpenOrders('BTCUSDT'), 'Problem in getting list of open orders with symbol.')

    def testGetFutureOrder(self):
        futuresOrderData = self.tradeGate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'MARKET', quantity=0.002)
        result = self.tradeGate.makeFuturesOrder(futuresOrderData)
        # self.log.info('\n\n{}'.format(result.orderId))
        order = self.tradeGate.getFuturesOrder('BTCUSDT', orderId=result.orderId)
        self.assertEqual(order.clientOrderId, result.clientOrderId)

        order = self.tradeGate.getFuturesOrder('BTCUSDT', localOrderId=result.clientOrderId)
        self.assertEqual(order.orderId, result.orderId)

    def testCancelingAllFuturesOpenOrders(self):
        futuresOrderData = self.tradeGate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'TAKE_PROFIT_MARKET', stopPrice=35000, quantity=0.002)
        result = self.tradeGate.makeFuturesOrder(futuresOrderData)

        result = self.tradeGate.cancellAllSymbolFuturesOrders('BTCUSDT', 1)

        openOrders = self.tradeGate.getAllFuturesOpenOrders('BTCUSDT')
        self.assertEqual(len(openOrders), 0, 'Problem in canceling all Open Orders')

    def testCancelingOrder(self):
        futuresOrderData = self.tradeGate.createAndTestFuturesOrder('BTCUSDT', 'BUY', 'TAKE_PROFIT_MARKET', stopPrice=35000, quantity=0.002)
        result = self.tradeGate.makeFuturesOrder(futuresOrderData)

        result = self.tradeGate.cancelFuturesOrder(symbol='BTCUSDT', localOrderId=result.clientOrderId)
        self.assertEqual(result.status, 'CANCELED', 'Problem in canceling specified Open Orders')

if __name__ == '__main__':
    unittest.main()